@@ -1,9 +1,23 @@
 /*
- * New simplified depmod without backwards compat stuff and not
- * requiring ksyms.
+ * depmod.c: generate module dependency meta-data (aliases, etc.)
  *
- * (C) 2010 Jon Masters <jcm@jonmasters.org>, and others.
  * (C) 2002 Rusty Russell IBM Corporation
+ * (C) 2006-2011 Jon Masters <jcm@jonmasters.org>, and others.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 #define _GNU_SOURCE /* asprintf */
 
@@ -42,6 +56,7 @@
 #define MODULE_BUILTIN_KEY "built-in"
 #endif
 
+/* used to replace one module with another based on conf override entries */
 struct module_overrides
 {
 	/* Next override */
@@ -51,6 +66,7 @@ struct module_overrides
 	char *modfile;
 };
 
+/* used to specify the order in which /lib/modules subdirs are searched */
 struct module_search
 {
 	/* Next search */
@@ -61,7 +77,8 @@ struct module_search
 	size_t len;
 };
 
-static unsigned int skipchars;
+static char sym_prefix; /* used for -P option */
+static unsigned int skipchars; /* prefix target part of basedir to skip over */
 static unsigned int make_map_files = 1; /* default to on */
 static unsigned int force_map_files = 0; /* default to on */
 
@@ -76,7 +93,12 @@ struct symbol
 
 static struct symbol *symbolhash[SYMBOL_HASH_SIZE];
 
-/* This is based on the hash agorithm from gdbm, via tdb */
+/**
+ * tdb_hash - calculate hash entry for a symbol (algorithm from gdbm, via tdb)
+ *
+ * @name:	symbol name
+ *
+ */
 static inline unsigned int tdb_hash(const char *name)
 {
 	unsigned value;	/* Used to compute the hash value.  */
@@ -89,7 +111,26 @@ static inline unsigned int tdb_hash(const char *name)
 	return (1103515243 * value + 12345);
 }
 
-void add_symbol(const char *name, uint64_t ver, struct module *owner)
+/**
+ * skip_symprefix - remove extraneous prefix character on some architectures
+ *
+ * @symname:	symbol name to process for prefix character removal
+ *
+ */
+static const char *skip_symprefix(const char *symname)
+{
+	return symname + (symname[0] == sym_prefix ? 1 : 0);
+}
+
+/**
+ * add_symbol - add a symbol to the symbol hash list
+ *
+ * @name:	symbol name
+ * @ver:	symbol version (checksum)
+ * @owner:	module owning this symbol
+ *
+ */
+static void add_symbol(const char *name, uint64_t ver, struct module *owner)
 {
 	unsigned int hash;
 	struct symbol *new = NOFAIL(malloc(sizeof *new + strlen(name) + 1));
@@ -105,7 +146,20 @@ void add_symbol(const char *name, uint64_t ver, struct module *owner)
 
 static int print_unknown, check_symvers;
 
-struct module *find_symbol(const char *name, uint64_t ver,
+/**
+ * find_symbol - lookup module owning a symbol in the symbol hash list
+ *
+ * @name:	symbol name
+ * @ver:	symbol version
+ * @modname:	pathname of module requesting lookup (being processed)
+ * @weak:	whether the symbol is weakly defined or not
+ *
+ * This function is used during dependency calculation to match the
+ * dependencies a module says it requires with symbols we have seen.
+ * calculate_deps calls us after it has load_dep_syms on a module.
+ *
+ */
+static struct module *find_symbol(const char *name, uint64_t ver,
 		const char *modname, int weak)
 {
 	struct symbol *s;
@@ -113,6 +167,7 @@ struct module *find_symbol(const char *name, uint64_t ver,
 	/* For our purposes, .foo matches foo.  PPC64 needs this. */
 	if (name[0] == '.')
 		name++;
+	name = skip_symprefix(name);
 
 	for (s = symbolhash[tdb_hash(name) % SYMBOL_HASH_SIZE]; s; s=s->next) {
 		if (streq(s->name, name))
@@ -131,7 +186,14 @@ struct module *find_symbol(const char *name, uint64_t ver,
 	return NULL;
 }
 
-void add_dep(struct module *mod, struct module *depends_on)
+/**
+ * add_dep - add a module dependency
+ *
+ * @mod:	module name
+ * @depends_on:	new module dependency
+ *
+ */
+static void add_dep(struct module *mod, struct module *depends_on)
 {
 	unsigned int i;
 
@@ -143,6 +205,12 @@ void add_dep(struct module *mod, struct module *depends_on)
 	mod->deps[mod->num_deps++] = depends_on;
 }
 
+/**
+ * add_fake_syms - symbols not explicitly otherwise provided
+ *
+ * The kernel provides a few dependencies within the module loader.
+ *
+ */
 static void add_fake_syms(void)
 {
 	/* __this_module is magic inserted by kernel loader. */
@@ -151,6 +219,12 @@ static void add_fake_syms(void)
 	add_symbol("_GLOBAL_OFFSET_TABLE_", 0, NULL);
 }
 
+/**
+ * load_system_map - load list of symbols from the System.map
+ *
+ * @filename:	path to the System.map-like file
+ *
+ */
 static void load_system_map(const char *filename)
 {
 	FILE *system_map;
@@ -165,6 +239,7 @@ static void load_system_map(const char *filename)
 	/* eg. c0294200 R __ksymtab_devfs_alloc_devnum */
 	while (fgets(line, sizeof(line)-1, system_map)) {
 		char *ptr;
+		const char *cptr;
 
 		/* Snip \n */
 		ptr = strchr(line, '\n');
@@ -175,15 +250,24 @@ static void load_system_map(const char *filename)
 		if (!ptr || !(ptr = strchr(ptr + 1, ' ')))
 			continue;
 
+		/* Skip the space before symbol name */
+		cptr = skip_symprefix(ptr + 1);
+
 		/* Covers gpl-only and normal symbols. */
-		if (strstarts(ptr+1, ksymstr))
-			add_symbol(ptr+1+ksymstr_len, 0, NULL);
+		if (strstarts(cptr, ksymstr))
+			add_symbol(cptr + ksymstr_len, 0, NULL);
 	}
 
 	fclose(system_map);
 	add_fake_syms();
 }
 
+/**
+ * load_module_symvers - load list of symbol versions from the module.symvers
+ *
+ * @filename:	path to the module.symvers-like file
+ *
+ */
 static void load_module_symvers(const char *filename)
 {
 	FILE *module_symvers;
@@ -204,14 +288,14 @@ static void load_module_symvers(const char *filename)
 			continue;
 
 		if (streq(where, "vmlinux"))
-			add_symbol(sym, strtoull(ver, NULL, 16), NULL);
+			add_symbol(skip_symprefix(sym), strtoull(ver, NULL, 16), NULL);
 	}
 
 	fclose(module_symvers);
 	add_fake_syms();
 }
 
-static struct option options[] = { { "all", 0, NULL, 'a' },
+static const struct option options[] = { { "all", 0, NULL, 'a' },
 				   { "quick", 0, NULL, 'A' },
 				   { "basedir", 1, NULL, 'b' },
 				   { "config", 1, NULL, 'C' },
@@ -224,23 +308,35 @@ static struct option options[] = { { "all", 0, NULL, 'a' },
 				   { "verbose", 0, NULL, 'v' },
 				   { "show", 0, NULL, 'n' },
 				   { "dry-run", 0, NULL, 'n' },
+				   { "symbol-prefix", 0, NULL, 'P' },
 				   { "help", 0, NULL, 'h' },
 				   { "version", 0, NULL, 'V' },
 				   { "warn", 0, NULL, 'w' },
 				   { "map", 0, NULL, 'm' },
 				   { NULL, 0, NULL, 0 } };
 
-/* Version number or module name?  Don't assume extension. */
+/**
+ * is_version_number - is the option a kernel version or module name
+ *
+ * @version:	possible version number
+ *
+ */
 static int is_version_number(const char *version)
 {
 	unsigned int dummy;
 
-	return (sscanf(version, "%u.%u.%u", &dummy, &dummy, &dummy) == 3);
+	return (sscanf(version, "%u.%u", &dummy, &dummy) == 2);
 }
 
+/**
+ * old_module_version - is the kernel version too old for these tools
+ *
+ * @version:	version number
+ *
+ */
 static int old_module_version(const char *version)
 {
-	/* Expect three part version. */
+	/* Expect three part version (but won't fail it only two part). */
 	unsigned int major, sub, minor;
 
 	sscanf(version, "%u.%u.%u", &major, &sub, &minor);
@@ -257,6 +353,12 @@ static int old_module_version(const char *version)
 	return 1;
 }
 
+/**
+ * print_usage - output a list of all possible options
+ *
+ * @name: not currently used
+ *
+ */
 static void print_usage(const char *name)
 {
 	fprintf(stderr,
@@ -275,6 +377,7 @@ static void print_usage(const char *name)
 	"\t-e, --errsyms        Report not supplied symbols\n"
 	"\t-m, --map            Create the legacy map files\n"
 	"\t-n, --show           Write the dependency file on stdout only\n"
+	"\t-P, --symbol-prefix  Architecture symbol prefix\n"
 	"\t-V, --version        Print the release version\n"
 	"\t-v, --verbose        Enable verbose mode\n"
 	"\t-w, --warn		Warn on duplicates\n"
@@ -292,6 +395,13 @@ static void print_usage(const char *name)
 	"depmod", "depmod");
 }
 
+/**
+ * ends_in - check file extension
+ *
+ * @name:	filename
+ * @ext:	extension to check
+ *
+ */
 static int ends_in(const char *name, const char *ext)
 {
 	unsigned int namelen, extlen;
@@ -307,6 +417,13 @@ static int ends_in(const char *name, const char *ext)
 	return 0;
 }
 
+/**
+ * grab_module - open module ELF file and load symbol data
+ *
+ * @dirname:	path prefix
+ * @filename:	filename within path
+ *
+ */
 static struct module *grab_module(const char *dirname, const char *filename)
 {
 	struct module *new;
@@ -332,12 +449,20 @@ static struct module *grab_module(const char *dirname, const char *filename)
 	return new;
 }
 
+/* We use this on-stack structure to track recursive calls to has_dep_loop */
 struct module_traverse
 {
 	struct module_traverse *prev;
 	struct module *mod;
 };
 
+/**
+ * in_loop - determine if a module dependency loop exists
+ *
+ * @mod:	current module
+ * @traverse:	on-stack structure created as module deps were processed
+ *
+ */
 static int in_loop(struct module *mod, const struct module_traverse *traverse)
 {
 	const struct module_traverse *i;
@@ -349,7 +474,13 @@ static int in_loop(struct module *mod, const struct module_traverse *traverse)
 	return 0;
 }
 
-/* Assume we are doing all the modules, so only report each loop once. */
+/**
+ * report_loop - report (once) that a dependency loop exists for a module
+ *
+ * @mod:	module with dep loop
+ * @traverse:	on-stack structure created as module deps were processed
+ *
+ */
 static void report_loop(const struct module *mod,
 			const struct module_traverse *traverse)
 {
@@ -374,8 +505,20 @@ static void report_loop(const struct module *mod,
 	fprintf(stderr, "which needs %s again!\n", i->mod->basename);
 }
 
-/* This is damn slow, but loops actually happen, and we don't want to
-   just exit() and leave the user without any modules. */
+/**
+ * has_dep_loop - iterate over all module deps and check for loops
+ *
+ * @module:	module to process
+ * @prev:	previously processed dependency
+ *
+ * This function is called recursively, following every dep and creating
+ * a module_traverse on the stack describing each dependency encountered.
+ * Determining a loop is as simple (and slow) as finding repetitions.
+ *
+ * This is slow, but we can't leave the user without any modules, so we
+ * need to detect loops and just fail those modules that cause loops.
+ *
+ */
 static int has_dep_loop(struct module *module, struct module_traverse *prev)
 {
 	unsigned int i;
@@ -392,7 +535,16 @@ static int has_dep_loop(struct module *module, struct module_traverse *prev)
 	return 0;
 }
 
-/* Uniquifies and orders a dependency list. */
+/**
+ * order_dep_list - expand all module deps recursively and in order
+ *
+ * @start:	module being processed
+ * @mod:	recursive dep
+ *
+ * We expand all of the dependencies of the dependencies of a module
+ * and ensure that the lowest dependency is loaded first, etc.
+ *
+ */
 static void order_dep_list(struct module *start, struct module *mod)
 {
 	unsigned int i;
@@ -409,6 +561,13 @@ static void order_dep_list(struct module *start, struct module *mod)
 
 static struct module *deleted = NULL;
 
+/**
+ * del_module - remove from list of modules
+ *
+ * @modules:	list of modules
+ * @delme:	module to remove
+ *
+ */
 static void del_module(struct module **modules, struct module *delme)
 {
 	struct module **i;
@@ -426,7 +585,17 @@ static void del_module(struct module **modules, struct module *delme)
 	deleted = delme;
 }
 
-/* convert to relative path if possible */
+/**
+ * compress_path - strip out common path prefix for modules
+ *
+ * @path:	path to module
+ * @basedir:	top level modules directory
+ *
+ * Modules are typically located in /lib/modules. There is no need to
+ * store the same common path prefix for all modules - make paths
+ * relative to directory that contains the dep information files.
+ *
+ */
 static const char *compress_path(const char *path, const char *basedir)
 {
 	int len = strlen(basedir);
@@ -436,6 +605,14 @@ static const char *compress_path(const char *path, const char *basedir)
 	return path;
 }
 
+/**
+ * output_deps - create ascii text file representation of module deps
+ *
+ * @modules:	list of modules
+ * @out:	output file
+ * @dirname:	output directory
+ *
+ */
 static int output_deps(struct module *modules,
 			FILE *out, char *dirname)
 {
@@ -459,8 +636,19 @@ static int output_deps(struct module *modules,
 }
 
 /* warn whenever duplicate module aliases, deps, or symbols are found. */
-int warn_dups = 0;
+static int warn_dups = 0;
 
+/**
+ * output_deps_bin - create binary trie representation of module deps
+ *
+ * @modules:	list of modules
+ * @out:	output binary file
+ * @dirname:	output directory
+ *
+ * This optimized dependency file contains an ordered structure that is
+ * more easily processed by modprobe in a time sensitive manner.
+ *
+ */
 static int output_deps_bin(struct module *modules,
 			FILE *out, char *dirname)
 {
@@ -502,7 +690,12 @@ static int output_deps_bin(struct module *modules,
 	return 1;
 }
 
-
+/**
+ * smells_like_module - detect common module extensions
+ *
+ * @name:	filename
+ *
+ */
 static int smells_like_module(const char *name)
 {
 	return ends_in(name,".ko") || ends_in(name, ".ko.gz");
@@ -514,6 +707,19 @@ typedef struct module *(*do_module_t)(const char *dirname,
 				      struct module_search *search,
 				      struct module_overrides *overrides);
 
+/**
+ * is_higher_priority - find modules replacing other modules
+ *
+ * @newpath:	new module path
+ * @oldpath:	old module path
+ * @search:	path search order
+ * @overrides:	module override directives
+ *
+ * Compares one module (path) to another module (path) and determines
+ * whether the new module should replace the existing module of the
+ * same name. Overriding is handled very coarsely for the moment.
+ *
+ */
 static int is_higher_priority(const char *newpath, const char *oldpath,
 			      struct module_search *search,
 			      struct module_overrides *overrides)
@@ -550,7 +756,16 @@ static int is_higher_priority(const char *newpath, const char *oldpath,
 	return prio_new > prio_old;
 }
 
-
+/**
+ * do_module - process a module file
+ *
+ * @dirname:	directory containing module
+ * @filename:	module disk file
+ * @list:	list of modules
+ * @search:	path search order
+ * @overrides:	module override directives
+ *
+ */
 static struct module *do_module(const char *dirname,
 				       const char *filename,
 				       struct module *list,
@@ -572,6 +787,8 @@ static struct module *do_module(const char *dirname,
 
 			sprintf(newpath, "%s/%s", dirname, filename);
 
+			/* if module matches an existing entry (name) but */
+			/* has a higher priority, replace existing entry. */
 			if (is_higher_priority(newpath, (*i)->pathname,search,
 					       overrides)) {
 				del_module(i, *i);
@@ -590,6 +807,16 @@ static struct module *do_module(const char *dirname,
 	return new;
 }
 
+/**
+ * grab_dir - process a directory of modules
+ *
+ * @dirname:	directory name
+ * @dir:	open directory reference
+ * @do_mod:	do_module function to use
+ * @search:	path search order
+ * @overrides:	module overrides directives
+ *
+ */
 static struct module *grab_dir(const char *dirname,
 			       DIR *dir,
 			       struct module *next,
@@ -623,6 +850,14 @@ static struct module *grab_dir(const char *dirname,
 	return next;
 }
 
+/**
+ * grab_basedir - top-level module processing
+ *
+ * @dirname:	top-level directory name
+ * @search:	path search order
+ * @overrides:	module overrides directives
+ *
+ */
 static struct module *grab_basedir(const char *dirname,
 				   struct module_search *search,
 				   struct module_overrides *overrides)
@@ -642,6 +877,17 @@ static struct module *grab_basedir(const char *dirname,
 	return list;
 }
 
+/**
+ * sort_modules - order modules in list on modules.order if available
+ *
+ * @dirname:	directory name
+ * @list:	module list
+ *
+ * Using the modules.order file (if available), give every module an index
+ * based on its position in the file, and order list based on the index. If
+ * no ordering data is available, fallback to existing unordered list.
+ *
+ */
 static struct module *sort_modules(const char *dirname, struct module *list)
 {
 	struct module *tlist = NULL, **tpos = &tlist;
@@ -693,7 +939,12 @@ static struct module *sort_modules(const char *dirname, struct module *list)
 	return tlist;
 }
 
-/* Calculate the dependencies for this module */
+/**
+ * calculate_deps - calculate deps for module
+ *
+ * @module:	module to process
+ *
+ */
 static void calculate_deps(struct module *module)
 {
 	unsigned int i;
@@ -734,6 +985,15 @@ static void calculate_deps(struct module *module)
 	free(symvers);
 }
 
+/**
+ * parse_modules - process the modules list
+ *
+ * @module:	module list
+ *
+ * Process each module in the (sorted by sort_modules) list for symbols,
+ * dependencies, and other meta-data that will be output later.
+ *
+ */
 static struct module *parse_modules(struct module *list)
 {
 	struct module *i;
@@ -748,7 +1008,7 @@ static struct module *parse_modules(struct module *list)
 				check_symvers ? &symvers : NULL);
 		if (syms) {
 			for (j = 0; j < syms->cnt; j++)
-				add_symbol(syms->str[j],
+				add_symbol(skip_symprefix(syms->str[j]),
 					symvers ? symvers[j] : 0, i);
 			strtbl_free(syms);
 		}
@@ -773,7 +1033,17 @@ static struct module *parse_modules(struct module *list)
 	return list;
 }
 
-/* Simply dump hash table. */
+/**
+ * output_symbols - output symbol alias information
+ *
+ * @unused:	unused
+ * @out:	output file reference
+ * @dirname:	output directory
+ *
+ * Output the symbol hash table, in the form of symbol alias entries, to
+ * an ascii text file of the form modules.symbols (depending on file).
+ *
+ */
 static int output_symbols(struct module *unused, FILE *out, char *dirname)
 {
 	unsigned int i;
@@ -794,6 +1064,17 @@ static int output_symbols(struct module *unused, FILE *out, char *dirname)
 	return 1;
 }
 
+/**
+ * output_symbols_bin - output symbol alias information in binary format
+ *
+ * @unused:	unused
+ * @out:	output file reference
+ * @dirname:	output directory
+ *
+ * Output the symbol hash table, in the form of symbol alias entries, to
+ * a trie ordered output file e.g. of the form modules.symbols.bin.
+ *
+ */
 static int output_symbols_bin(struct module *unused, FILE *out, char *dirname)
 {
 	struct index_node *index;
@@ -827,6 +1108,14 @@ static int output_symbols_bin(struct module *unused, FILE *out, char *dirname)
 	return 1;
 }
 
+/**
+ * output_builtin_bin - output list of built-in modules in binary format
+ *
+ * @unused:	unused
+ * @out:	output file reference
+ * @dirname:	output directory
+ *
+ */
 static int output_builtin_bin(struct module *unused, FILE *out, char *dirname)
 {
 	struct index_node *index;
@@ -863,6 +1152,14 @@ static int output_builtin_bin(struct module *unused, FILE *out, char *dirname)
 	return 1;
 }
 
+/**
+ * output_aliases - output list of module aliases
+ *
+ * @modules:	list of modules
+ * @out:	output file reference
+ * @dirname:	output directory
+ *
+ */
 static int output_aliases(struct module *modules, FILE *out, char *dirname)
 {
 	struct module *i;
@@ -896,6 +1193,14 @@ static int output_aliases(struct module *modules, FILE *out, char *dirname)
 	return 1;
 }
 
+/**
+ * output_aliases_bin - output list of module aliases in binary format
+ *
+ * @modules:	list of modules
+ * @out:	outout file reference
+ * @dirname:	output directory
+ *
+ */
 static int output_aliases_bin(struct module *modules, FILE *out, char *dirname)
 {
 	struct module *i;
@@ -950,6 +1255,14 @@ static int output_aliases_bin(struct module *modules, FILE *out, char *dirname)
 	return 1;
 }
 
+/**
+ * output_softdeps - output module softdeps (non-implicit dependencies)
+ *
+ * @modules:	list of modules
+ * @out:	output file reference
+ * @dirname:	output directory
+ *
+ */
 static int output_softdeps(struct module *modules, FILE *out, char *dirname)
 {
 	struct module *i;
@@ -979,6 +1292,14 @@ static int output_softdeps(struct module *modules, FILE *out, char *dirname)
 	return 1;
 }
 
+/**
+ * output_devname - output device names required by modules
+ *
+ * @modules:	list of modules
+ * @out:	output file reference
+ * @dirname:	output directory
+ *
+ */
 static int output_devname(struct module *modules, FILE *out, char *dirname)
 {
 	struct module *m;
@@ -1017,12 +1338,13 @@ static int output_devname(struct module *modules, FILE *out, char *dirname)
 }
 
 struct depfile {
-	char *name;
+	const char *name;
 	int (*func)(struct module *, FILE *, char *dirname);
 	int map_file;
 };
 
-static struct depfile depfiles[] = {
+/* The possible output files - those with map_file unset typically not made */
+static const struct depfile depfiles[] = {
 	{ "modules.dep", output_deps, 0 }, /* This is what we check for '-A'. */
 	{ "modules.dep.bin", output_deps_bin, 0 },
 	{ "modules.pcimap", output_pci_table, 1 },
@@ -1042,7 +1364,16 @@ static struct depfile depfiles[] = {
 	{ "modules.devname", output_devname, 0 },
 };
 
-/* If we can't figure it out, it's safe to say "true". */
+/**
+ * any_modules_newer - determine if modules are newer than ref time
+ *
+ * @dirname:	directory to process
+ * @mtime:	comparison time
+ *
+ * The worst case is that we process modules we didn't need to. It is
+ * therefore safer to go with "true" if we can't figure it out.
+ *
+ */
 static int any_modules_newer(const char *dirname, time_t mtime)
 {
 	DIR *dir;
@@ -1079,6 +1410,14 @@ ret_true:
 	return 1;
 }
 
+/**
+ * depfile_out_of_date - check if module dep files are older than any modules
+ *
+ * @dirname:	directory to process
+ *
+ * Use any_modules_newer to determine if the dep files are up to date.
+ *
+ */
 static int depfile_out_of_date(const char *dirname)
 {
 	struct stat st;
@@ -1092,6 +1431,13 @@ static int depfile_out_of_date(const char *dirname)
 	return any_modules_newer(dirname, st.st_mtime);
 }
 
+/**
+ * strsep_skipspace - skip over delimitors in strings
+ *
+ * @string:	string to process
+ * @delim:	delimitor (e.g. ' ')
+ *
+ */
 static char *strsep_skipspace(char **string, char *delim)
 {
 	if (!*string)
@@ -1100,6 +1446,14 @@ static char *strsep_skipspace(char **string, char *delim)
 	return strsep(string, delim);
 }
 
+/**
+ * add_search - add a new module search path
+ *
+ * @search_path:	path to search
+ * @len:		length of path
+ * @search:		list of search paths
+ *
+ */
 static struct module_search *add_search(const char *search_path,
 					size_t len,
 					struct module_search *search)
@@ -1116,6 +1470,13 @@ static struct module_search *add_search(const char *search_path,
 	
 }
 
+/**
+ * add_override - add a new module override entry
+ *
+ * @modfile:	name of module file
+ * @overrides:	list of override entries
+ *
+ */
 static struct module_overrides *add_override(const char *modfile,
 					     struct module_overrides *overrides)
 {
@@ -1136,6 +1497,16 @@ static int parse_config_scan(const char *filename,
 			     struct module_search **search,
 			     struct module_overrides **overrides);
 
+/**
+ * parse_config_file - process an individual configuration file
+ *
+ * @filename:		name of config file
+ * @basedir:		module base directory
+ * @kernelversion:	kernel version to process
+ * @search:		search path order
+ * @overrides:		module override entries
+ *
+ */
 static int parse_config_file(const char *filename,
 			     const char *basedir,
 			     const char *kernelversion,
@@ -1242,6 +1613,16 @@ static int parse_config_file(const char *filename,
         return 1;
 }
 
+/**
+ * parse_config_scan - handle a directory of config files
+ *
+ * @filename:		name of directory
+ * @basedir:		module base directory
+ * @kernelversion:	kernel version to process
+ * @search:		search path order
+ * @overrides:		module override entries
+ *
+ */
 static int parse_config_scan(const char *filename,
 			     const char *basedir,
 			     const char *kernelversion,
@@ -1310,6 +1691,16 @@ static int parse_config_scan(const char *filename,
 	return ret;
 }
 
+/**
+ * parse_toplevel_config - handle top-level depmod.conf, depmod.d
+ *
+ * @filename:		possibly overridden config
+ * @basedir:		module base directory
+ * @kernelversion:	kernel version to process
+ * @search:		search path order
+ * @overrides:		module override entries
+ *
+ */
 static void parse_toplevel_config(const char *filename,
 				  const char *basedir,
 				  const char *kernelversion,
@@ -1336,9 +1727,9 @@ static void parse_toplevel_config(const char *filename,
 }
 
 /* Local to main, but not freed on exit.  Keep valgrind quiet. */
-struct module *list = NULL;
-struct module_search *search = NULL;
-struct module_overrides *overrides = NULL;
+static struct module *list = NULL;
+static struct module_search *search = NULL;
+static struct module_overrides *overrides = NULL;
 
 int main(int argc, char *argv[])
 {
@@ -1351,7 +1742,7 @@ int main(int argc, char *argv[])
 	if (native_endianness() == 0)
 		abort();
 
-	while ((opt = getopt_long(argc, argv, "aAb:C:E:F:euqrvnhVwm", options, NULL))
+	while ((opt = getopt_long(argc, argv, "aAb:C:E:F:euqrvnP:hVwm", options, NULL))
 	       != -1) {
 		switch (opt) {
 		case 'a':
@@ -1387,6 +1778,11 @@ int main(int argc, char *argv[])
 		case 'n':
 			doing_stdout = 1;
 			break;
+		case 'P':
+			if (optarg[1] != '\0')
+				fatal("-P only takes a single char\n");
+			sym_prefix = optarg[0];
+			break;
 		case 'h':
 			print_usage(argv[0]);
 			exit(0);
@@ -1481,10 +1877,10 @@ int main(int argc, char *argv[])
 	list = sort_modules(dirname,list);
 	list = parse_modules(list);
 
-	for (i = 0; i < sizeof(depfiles)/sizeof(depfiles[0]); i++) {
+	for (i = 0; i < ARRAY_SIZE(depfiles); i++) {
 		FILE *out;
 		int res;
-		struct depfile *d = &depfiles[i];
+		const struct depfile *d = &depfiles[i];
 		char depname[strlen(dirname) + 1 + strlen(d->name) + 1];
 		char tmpname[strlen(dirname) + 1 + strlen(d->name) +
 						strlen(".temp") + 1];
