@@ -28,9 +28,8 @@
 #include <errno.h>
 #include <asm/unistd.h>
 
-#include "backwards_compat.c"
-
-#define streq(a,b) (strcmp((a),(b)) == 0)
+#include "util.h"
+#include "testing.h"
 
 extern long init_module(void *, unsigned long, const char *);
 
@@ -60,8 +59,8 @@ static const char *moderror(int err)
 static void *grab_file(const char *filename, unsigned long *size)
 {
 	unsigned int max = 16384;
-	int ret, fd;
-	void *buffer = malloc(max);
+	int ret, fd, err_save;
+	void *buffer;
 
 	if (streq(filename, "-"))
 		fd = dup(STDIN_FILENO);
@@ -71,18 +70,34 @@ static void *grab_file(const char *filename, unsigned long *size)
 	if (fd < 0)
 		return NULL;
 
+	buffer = malloc(max);
+	if (!buffer)
+		goto out_error;
+
 	*size = 0;
 	while ((ret = read(fd, buffer + *size, max - *size)) > 0) {
 		*size += ret;
-		if (*size == max)
-			buffer = realloc(buffer, max *= 2);
-	}
-	if (ret < 0) {
-		free(buffer);
-		buffer = NULL;
+		if (*size == max) {
+			void *p;
+
+			p = realloc(buffer, max *= 2);
+			if (!p)
+				goto out_error;
+			buffer = p;
+		}
 	}
+	if (ret < 0)
+		goto out_error;
+
 	close(fd);
 	return buffer;
+
+out_error:
+	err_save = errno;
+	free(buffer);
+	close(fd);
+	errno = err_save;
+	return NULL;
 }
 
 int main(int argc, char *argv[])
@@ -92,12 +107,16 @@ int main(int argc, char *argv[])
 	unsigned long len;
 	void *file;
 	char *filename, *options = strdup("");
-	char *progname = argv[0];
+	char *p, *progname = argv[0];
 
-	if (strstr(argv[0], "insmod.static"))
-		try_old_version("insmod.static", argv);
-	else
-		try_old_version("insmod", argv);
+	if (!options) {
+		fprintf(stderr,
+			"insmod: can't allocate memory: %s\n",
+			strerror(errno));
+		exit(1);
+	}
+
+	p = my_basename(argv[0]);
 
 	if (argv[1] && (streq(argv[1], "--version") || streq(argv[1], "-V"))) {
 		puts(PACKAGE " version " VERSION);
@@ -120,6 +139,12 @@ int main(int argc, char *argv[])
 	for (i = 2; i < argc; i++) {
 		options = realloc(options,
 				  strlen(options) + 1 + strlen(argv[i]) + 1);
+		if (!options) {
+			fprintf(stderr,
+				"insmod: can't allocate memory: %s\n",
+				strerror(errno));
+			exit(1);
+		}
 		strcat(options, argv[i]);
 		strcat(options, " ");
 	}
@@ -135,7 +160,10 @@ int main(int argc, char *argv[])
 	if (ret != 0) {
 		fprintf(stderr, "insmod: error inserting '%s': %li %s\n",
 			filename, ret, moderror(errno));
-		exit(1);
 	}
+	free(file);
+
+	if (ret != 0)
+		exit(1);
 	exit(0);
 }
